/***************************************************************************
                          apollontransferviewitem.cpp  -  description
                             -------------------
begin		: Dec 05 2003
copyright	: (C) 2002 by Bastiaan Naber, (C) 2002 by Robert Wittams
email		: bastiaan@ricardis.tudelft.nl
description	: 

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <assert.h>
#include <iostream>
using namespace std;

#include <kglobal.h>
#include <kiconloader.h>
#include <klocale.h>
#include <qpainter.h>
#include <qpen.h>
#include <kpixmapeffect.h>

#include "apollontransferviewitem.h"

ApollonTransferViewItem::ApollonTransferViewItem(QListView *parent, const QString &updown, int ID, const QString &protocol_name) :
  QListViewItem(parent),
  m_ID(ID),
  m_hash(""),
  m_name(""),
  m_size(-1),
  m_transmit(-1),
  m_bandwidth(""),
  m_status(""),
  m_eta(-1),
  m_updateString(""),
  m_last_width(0)
{
  m_Updated = FALSE;
  updateIcon(protocol_name, updown);
  m_Updated = FALSE;
  m_isPaused = FALSE;
  KIconLoader *loader = KGlobal::iconLoader();
  m_PausePix = loader->loadIcon("player_pause", KIcon::Small);
  m_protocolName = protocol_name;

}

ApollonTransferViewItem::ApollonTransferViewItem(QListViewItem *parent, int ID) :
  QListViewItem(parent),
  m_ID(ID),
  m_hash(""),
  m_name(""),
  m_size(-1),
  m_transmit(-1),
  m_bandwidth(""),
  m_status(""),
  m_eta(-1),
  m_updateString("")
{
  m_Updated = FALSE;
  m_isPaused = FALSE;
  KIconLoader *loader = KGlobal::iconLoader();
  m_PausePix = loader->loadIcon("player_pause", KIcon::Small);
}

ApollonTransferViewItem::~ApollonTransferViewItem()
{
}

void ApollonTransferViewItem::setStatus(QString status)
{
	m_status = status;

	KIconLoader *loader = KGlobal::iconLoader();

	if(m_status == i18n("Completed"))
	{
		setPixmap(0, loader->loadIcon("ok", KIcon::Small));
		m_bandwidth = "";
		m_eta = -1;
	}
	else if(m_status == i18n("Canceled"))
	{
		setPixmap(0, loader->loadIcon("no", KIcon::Small));
		m_bandwidth = "";
		m_eta = -1;
	}
	m_last_width=-1; // This generates a new gradient for the progress bar
}

ApollonTransferViewItem *ApollonTransferViewItem::getItem(const QListView *lv, int ID)
{
	assert(lv != 0);

	ApollonTransferViewItem *item = (ApollonTransferViewItem*) lv->firstChild();
	while(item != 0)
	{
		if(item->id() == ID)
		{
			break;
		}
		item = item->nextSibling();
	}

	return item;
}


void ApollonTransferViewItem::updateIcon(const QString & protocol_name, const QString & updown)
{
	if(m_Updated)
	{
		return;
	}

	KIconLoader *loader = KGlobal::iconLoader();

	if(updown == "download")
	{
		if(protocol_name=="FastTrack")
		{
			mIcon = "kazaadown";
		}
		else if(protocol_name=="Gnutella")
		{
			mIcon = "gnutelladown";
		}
		else if(protocol_name=="OpenFT")
		{
			mIcon = "openftdown";
		}
		else if(protocol_name=="OpenNap")
		{
			mIcon = "napsterdown";
		}
		else if(protocol_name=="SoulSeek")
		{
			mIcon = "soulseekdown";
		}
		else
		{
			mIcon = "1downarrow";
		}
	}
	else if(updown == "upload")
	{
		if(protocol_name=="FastTrack")
		{
			mIcon = "kazaaup";
		}
		else if(protocol_name=="Gnutella")
		{
			mIcon = "gnutellaup";
		}
		else if(protocol_name=="OpenFT")
		{
			mIcon = "openftup";
		}
		else if(protocol_name=="SoulSeek")
		{
			mIcon = "soulseekup";
		}
		else
		{
			mIcon = "1uparrow";
		}
	}

	m_Pix = loader->loadIcon(mIcon, KIcon::Small);
	setPixmap(0,m_Pix);
	m_Updated = TRUE;
}

void ApollonTransferViewItem::pauseItem() {
  setPixmap(0, m_PausePix);
  m_isPaused = TRUE;
  m_Updated = TRUE; //don't change the icon....
}

void ApollonTransferViewItem::unpauseItem() {
  setPixmap(0, m_Pix);
  m_isPaused = FALSE;
  m_Updated = TRUE;
}
QString ApollonTransferViewItem::key(int column, bool /*ascending*/) const
{
	QString temp1;
	switch(column)
	{
		case 1:
			temp1 = temp1.setNum(size());
			return temp1;
			break;
		case 2:
			temp1 = temp1.setNum(transmit());
			return temp1;
			break;
		case 3:
			temp1 = bandwidth();
			if(temp1=="--") return "0.00";
			temp1 = temp1.remove(" KB/s");
			return temp1;
			break;
		case 5:
			temp1 = temp1.setNum(eta());
			return temp1;
			break;
		case columnStatus:
			if (m_progress != -1)
				temp1.setNum((int)m_progress);
			else
				temp1 = m_status;
			return temp1;
			break;
		default:
			return text(column);
			break;
	}
}
int ApollonTransferViewItem::compare( QListViewItem* item, int column, bool ascending ) const
{
	if ( column != 0  ) {
	QString current = key( column, ascending);
	QString next = item->key( column, ascending);
	if((current.toDouble() - next.toDouble())>0)
	return 1;
	else return -1;
	} else {
	return QListViewItem::compare( item, column, ascending );
	}
	}


void ApollonTransferViewItem::paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align)
{
	// status bars
	if(column == columnStatus)
	{
		int fillwidth;
		p->setPen(cg.foreground());
		p->drawRect(1, 1, width - 2, height() - 2);
		p->setPen(QColor(255, 255, 255));
		p->drawRect(0, 0, width, height());
		p->drawRect(2, 2, width - 4, height() - 4);
		if(m_last_width != width)
		{
			m_last_width = width;
			m_gradient.resize(width, height());
			QColor color;
			if(m_progress != -1)
				color = cg.highlight();
			else
				color = cg.background();
			KPixmapEffect::gradient((KPixmap &) m_gradient, color, color.light(), KPixmapEffect::EllipticGradient);
		}
		if(parent() == 0)
		{
			if(m_progress == -1)
				fillwidth = width - 6;
			else
				fillwidth = (int) (m_progress / 100.0 * (width - 6));
			bitBlt(p->device(), (int) p->worldMatrix().dx() + 3, (int) p->worldMatrix().dy() + 3, &m_gradient, 0, 0, fillwidth, height() - 6, Qt::CopyROP, false);
			p->fillRect(3 + fillwidth, 3, width - fillwidth - 6, height() - 6, QColor(255, 255, 255));
			p->setPen(cg.foreground());
			if(m_progress != -1)
				p->drawText(0, 0, width, height(), Qt::AlignCenter, QString::number((int) m_progress) + "%");
			else
				p->drawText(0, 0, width, height(), Qt::AlignCenter, m_status);
		}
		else
		{
			fillwidth = (int) (m_progress / 100.0 * (width - 6));
			bitBlt(p->device(), (int) p->worldMatrix().dx() + 3, (int) p->worldMatrix().dy() + 3, &m_gradient, 0, 0, fillwidth, height() - 6, Qt::CopyROP, false);
			p->fillRect(3 + fillwidth, 3, width - fillwidth - 6, height() - 6, QColor(255, 255, 255));
			p->setPen(cg.foreground());
			p->drawText(0, 0, width, height(), Qt::AlignCenter, m_status);
		}
		listView()->repaint();
	}
	// other cells
	else
	{
		QListViewItem::paintCell(p, cg, column, width, align);
	}
}
